(***********************************************************************)
(*                                                                     *)
(*                                Ulib                                 *)
(*                                                                     *)
(*                   OCaml user's additional library                   *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2004-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: ustamped_darray.ml,v 1.7 2014/02/09 17:27:28 fclement Exp $ *)

(* Stamped dynamic arrays. *)

module Darray = Udynamic_array;;

let sda_qual = Upervasives.qualify "Ustamped_darray";;

let sda_invalid_arg = Upervasives.invalid_arg_qual sda_qual;;

type index = Darray.index;;
type length = Darray.length;;

type 'a stamped =
  | Unset
  | Valid of 'a
;;

exception Not_set;;

type 'a t =
  {
   darray : 'a stamped Darray.t;
   mutable next_unset : index;
  }
;;

let make n =
  {
   darray = Darray.make n Unset;
   next_unset = Unatural._0_;
  }
;;

let get sda i =
  match Darray.get sda.darray i with
  | Unset -> raise Not_set
  | Valid x -> x
;;

let set ({ darray = da; next_unset = n; } as sda) i x =
  if i > n then
    sda_invalid_arg "set"
      (Printf.sprintf "index %d is greater then next unset %d"
         (Unatural.to_int i) (Unatural.to_int n)) else
  begin Darray.set da i (Valid x);
  if i = n then sda.next_unset <- Unatural.succ i end
;;

let set_next sda x = set sda sda.next_unset x;;

let same sda1 sda2 =
  Darray.same sda1.darray sda2.darray &&
  sda1.next_unset = sda2.next_unset
;;

let max_length = Darray.max_length;;

let length sda = Darray.length sda.darray;;

(* Converters. *)

let get_valid = function
  | Unset -> assert false
  | Valid x -> x
;;

let to_array { darray = da; next_unset = n; } =
  Uarray.init n (fun i -> get_valid (Darray.get da (Unatural.of_int i)))
;;

let valid x = Valid x;;

let of_array a =
  let n = Uarray.length a in
  { darray = Darray.of_array Unset (Array.map valid a); next_unset = n; }
;;

(* Iterators. *)

let iter f { darray = da; _ } =
  let f i = function
    | Unset -> ()
    | Valid x -> f i x in
  Darray.iter f da
;;

let map f ({ darray = da; _ } as sda) =
  let f i = function
    | Unset -> Unset
    | Valid x -> Valid (f i x) in
  { sda with darray = Darray.map f da; }
;;

let fold_left f accu { darray = da; _ } =
  let f accu i = function
    | Unset -> accu
    | Valid x -> f accu i x in
  Darray.fold_left f accu da
;;
