(***********************************************************************)
(*                                                                     *)
(*                                Ulib                                 *)
(*                                                                     *)
(*                   OCaml user's additional library                   *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2004-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: ustring.mli,v 1.19 2014/01/10 15:28:04 fclement Exp $ *)

(** {3 More string operations} *)

type length = Upervasives.length;;

val length : string -> length;;
(** Return the length (number of characters) of the given string. *)

val create : length -> string;;
(** [create n] returns a fresh string of length [n].
   The string initially contains arbitrary characters.
   
   Raise [Invalid_argument] if [n > Sys.max_string_length]. *)

val make : length -> char -> string
(** [make n c] returns a fresh string of length [n], filled with the
   character [c].
   
   Raise [Invalid_argument] if [n > Sys.max_string_length]. *)

val init : length -> (int -> char) -> string;;
(** [init n f] returns a fresh string of length [n],
   with element number [i] initialized to the result of [f i].
   In other terms, [init n f] tabulates the results of [f]
   applied to the integers [0] to [n-1].

   Raise [Invalid_argument] if [n > Sys.max_string_length]. *)

val sub : string -> int -> length -> string
(** [sub s ofs len] returns a fresh string of length [len], containing the
   substring of [s] that starts at index [ofs] and has length [len].
   
   Raise [Invalid_argument] if [ofs] and [len] do not designate a valid
   substring of [s]. *)

val fill : string -> int -> length -> char -> unit
(** [fill s ofs len c] modifies string [s] in place, replacing [len]
   characters by [c], starting at index [ofs].
   
   Raise [Invalid_argument] if [ofs] and [len] do not designate a valid
   substring of [s]. *)

val blit : string -> int -> string -> int -> length -> unit
(** [blit src srcofs dst dstofs len] copies [len] characters from string [src],
   starting at index [srcofs], to string [dst], starting at index [dstofs].
   It works correctly even if [src] and [dst] are the same string, and the
   source and destination intervals overlap.
   
   Raise [Invalid_argument] if [srcofs] and [len] do not designate a valid
   substring of [src], or if [dstofs] and [len] do not designate a valid
   substring of [dst]. *)

val replace :
  string -> int -> length -> string -> int -> length -> string
;;
(** [replace s1 ofs1 len1 s2 ofs2 len2] returns a copy of [s1] with the
   substring starting at index [ofs1] and of length [len1] replaced by the
   substring of [s2] starting at index [ofs2] and of length [len2].
   Lengths [len1] and [len2] may differ. *)

val fold_left : ('a -> char -> 'a) -> 'a -> string -> 'a;;
(** [fold_left f a s] is
   [f (... (f (f a s.[0]) s.[1]) ...) s.[String.length s - 1]]. *)

val indices : string -> char -> int list;;
(** [indices s c] returns the list of indices of all the occurrences of
   character [c] in string [s]. *)

type prefix = string
and suffix = string
;;
(** The types for prefixes and suffixes;
   these are aliases for type [string]. *)

val chop_prefix : prefix -> string -> string;;
(** [chop_prefix pref s] returns [s] without prefix [pref].
   Returns [s], if [s] does not start with prefix [pref]. *)

val chop_suffix : suffix -> string -> string;;
(** [chop_suffix suff s] returns [s] without suffix [suff].
   Returns [s], if [s] does not end with suffix [suff]. *)

val trim_edges : (char -> bool) -> string -> string;;
(** [trim_edges p s] returns a copy of [s] without leading and
   trailing characters that satisfy predicate [p].
   Returns [s], if there is no such leading nor trailing characters in [s]. *)

val trim_consecutive : (char -> bool) -> string -> string;;
(** [trim_consecutive p s] returns a copy of [s] with no repetition of
   characters that satisfy predicate [p].
   If there is no such repetition, the original string [s] is returned. *)

val trim_doubles : string -> string;;
(** [trim_doubles s] returns a copy of [s] with no repeated characters.
   If there is no repetition, the original string [s] is returned. *)

val iso_latin1_to_us_ascii : (char -> string) -> string -> string;;
(** [iso_latin1_to_us_ascii string_of_unextended_char s] returns a copy of [s]
    with all non US ASCII ISO Latin-1 (8859-1) characters translated into
    US ASCII characters. Digrams are replaced by the two corresponding
    characters and non alphabetic non US ASCII characters are treated by the
    function [string_of_unextended_char]. *)

(** {6 Keys} *)

type key = string
and long_key = key
and short_key = key
;;
(** The types of keys, as to be used with modules Arg or Ucommand_option.
   Key strings are assumed to begin with a dash ['-'] character. *)

val long_key_of_string : string -> long_key;;
(** [long_key_of_string s] maps argument string [s] to the same string where
   all plain ascii alphabetic characters are replaced by their lowercase form,
   and all consecutive non plain ascii alphanumeric characters are replaced by
   a single dash ['-'] character. *)

val short_key_of_long_key : long_key -> short_key;;
(** [short_key_of_long_key lk] returns the key made of a dash ['-'] character
   followed by the initial of all dash-separated words of the argument long key
   [lk]. *)
